import argparse
import json

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser
from apigee.util import authorization, console, mfa_with_pyotp

from apigee.util import console


class ParserAuthorization:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_auth = self._parser.add_parser(
            "authorization", aliases=["auth"], help="check auth"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_auth(self):
        return self._parser_auth

    @parser_auth.setter
    def parser_auth(self, value):
        self._parser_auth = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    def __call__(self):
        return self._parser

    def authorization_with_prefix(self, auth, name, file=None, key="name"):
        if file:
            with open(file) as f:
                attr = json.loads(f.read())[key]
            return authorization.with_prefix(attr, auth)
        return authorization.with_prefix(name, auth)

    def _build_access_token_argument(self):
        parser = self._parser_auth.add_parser(
            "access",
            aliases=["access-token"],
            help="get access token",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.set_defaults(
            func=lambda args: console.log(mfa_with_pyotp.get_access_token(args))
        )

    def _build_verify_resource_file_argument(self):
        parser = self._parser_auth.add_parser(
            "file",
            aliases=["verify-resource-file"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._file_parser(),
            ],
            help="verify user has authorization to access resource with prefix in file",
        )
        parser.add_argument(
            "-k",
            "--key",
            action="store",
            help="key of attribute with prefix to verify",
            required=False,
            default="name",
        )
        parser.set_defaults(
            func=lambda args: console.log(
                self.authorization_with_prefix(args, None, file=args.file, key=args.key)
            )
        )

    def _build_verify_resource_name_argument(self):
        parser = self._parser_auth.add_parser(
            "name",
            aliases=["verify-resource-name"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="verify user has authorization to access resource with prefix in name",
        )
        parser.add_argument(
            "-n", "--name", help="name of resource with prefix to verify", required=True
        )
        parser.set_defaults(
            func=lambda args: console.log(
                self.authorization_with_prefix(args, args.name)
            )
        )

    def _create_parser(self):
        self._build_access_token_argument()
        self._build_verify_resource_file_argument()
        self._build_verify_resource_name_argument()



#!/usr/bin/env python
"""apigee"""

import argparse
import json

import apigee

from apigee.util import *

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.dir_parser import DirParser
from apigee.parsers.format_parser import FormatParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.parsers.parser_authorization import ParserAuthorization
from apigee.parsers.parser_configure import ParserConfigure
from apigee.parsers.parser_apis import ParserApis
from apigee.parsers.parser_deployments import ParserDeployments
from apigee.parsers.parser_keyvaluemaps import ParserKeyvaluemaps
from apigee.parsers.parser_developers import ParserDevelopers
from apigee.parsers.parser_apps import ParserApps
from apigee.parsers.parser_apiproducts import ParserApiproducts
from apigee.parsers.parser_targetservers import ParserTargetservers
from apigee.parsers.parser_maskconfigs import ParserMaskconfigs
from apigee.parsers.parser_permissions import ParserPermissions
from apigee.parsers.parser_userroles import ParserUserroles
from apigee.parsers.parser_caches import ParserCaches
from apigee.parsers.parser_debugsessions import ParserDebugsessions
from apigee.parsers.parser_sharedflows import ParserSharedflows
from apigee.parsers.parser_stats import ParserStats
from apigee.parsers.parser_backup import ParserBackup
from apigee.parsers.parser_restore import ParserRestore

from apigee.util.exceptions import exception_handler


@exception_handler
def main():
    parent_parser = ParentParser()
    file_parser = FileParser()
    dir_parser = DirParser()
    format_parser = FormatParser()
    environment_parser = EnvironmentParser()
    prefix_parser = PrefixParser(profile=parent_parser.profile)
    silent_parser = SilentParser()
    verbose_parser = VerboseParser()

    parser = argparse.ArgumentParser(prog=apigee.CMD, description=apigee.description)
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"{apigee.APP} {apigee.__version__}",
    )
    subparsers = parser.add_subparsers()

    ParserAuthorization(
        subparsers, silent_parser=silent_parser, verbose_parser=verbose_parser,
    ).parser
    ParserConfigure(subparsers).parser
    ParserApis(
        subparsers,
        parent_parser=parent_parser,
        dir_parser=dir_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserDeployments(
        subparsers,
        parent_parser=parent_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserKeyvaluemaps(
        subparsers,
        parent_parser=parent_parser,
        file_parser=file_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserDevelopers(
        subparsers,
        parent_parser=parent_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserApps(
        subparsers,
        parent_parser=parent_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserApiproducts(
        subparsers,
        parent_parser=parent_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserTargetservers(
        subparsers,
        parent_parser=parent_parser,
        file_parser=file_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserMaskconfigs(
        subparsers,
        parent_parser=parent_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserPermissions(
        subparsers,
        parent_parser=parent_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserUserroles(
        subparsers,
        parent_parser=parent_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserCaches(
        subparsers,
        parent_parser=parent_parser,
        file_parser=file_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserDebugsessions(
        subparsers,
        parent_parser=parent_parser,
        dir_parser=dir_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserSharedflows(
        subparsers,
        parent_parser=parent_parser,
        dir_parser=dir_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserStats(
        subparsers,
        parent_parser=parent_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserBackup(
        subparsers,
        parent_parser=parent_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser
    ParserRestore(
        subparsers,
        parent_parser=parent_parser,
        environment_parser=environment_parser,
        prefix_parser=prefix_parser,
        silent_parser=silent_parser,
        verbose_parser=verbose_parser,
    ).parser

    args = parser.parse_args()
    try:
        func = args.func
    except AttributeError:
        parser.error("too few arguments")
    func(args)


if __name__ == "__main__":
    main()



import functools
import logging
import sys

from apigee import *
from apigee.util import console
from apigee.util.os import touch


def exception_handler(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
            return result
        except Exception as e:
            touch(APIGEE_CLI_EXCEPTION_LOG_FILE)
            f_handler = logging.FileHandler(APIGEE_CLI_EXCEPTION_LOG_FILE, "w+")
            f_handler.setLevel(logging.INFO)
            formatter = logging.Formatter(
                "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
            )
            f_handler.setFormatter(formatter)
            logging.getLogger("").addHandler(f_handler)
            logging.error("Exception occurred", exc_info=True)
            sys.exit(
                f"An exception of type {type(e).__name__} occurred. Arguments:\n{e}"
            )
        except KeyboardInterrupt as ki:
            console.log()
            sys.exit(130)

    return wrapper


class CustomError(Exception):
    pass



import argparse

from apigee.api.sharedflows import Sharedflows

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console
from apigee.util.types import ArgumentType


class ParserSharedflows:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_sharedflows = self._parser.add_parser(
            "sharedflows", aliases=["sf"], help="manage shared flows"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_sharedflows(self):
        return self._parser_sharedflows

    @parser_sharedflows.setter
    def parser_sharedflows(self, value):
        self._parser_sharedflows = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_get_a_list_of_shared_flows_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "list",
            aliases=["get-a-list-of-shared-flows"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Gets an array of the names of shared flows in the organization. The response is a simple array of strings.",
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org).get_a_list_of_shared_flows(
                    prefix=args.prefix
                )
            )
        )

    def _build_import_a_shared_flow_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "import",
            aliases=["import-a-shared-flow"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                # self._file_parser(),
            ],
            help="Uploads a ZIP-formatted shared flow configuration bundle from a local machine to an Edge organization. If the shared flow already exists, this creates a new revision of it. If the shared flow does not exist, this creates it. Once imported, the shared flow revision must be deployed before it can be accessed at runtime. By default, shared flow configurations are not validated on import.",
        )
        parser.add_argument(
            "-f",
            "--file",
            action="store",
            help="file path of the shared flow configuration in ZIP format",
            required=True,
            type=ArgumentType.isfile,
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org)
                .import_a_shared_flow(args.file, args.name)
                .text
            )
        )

    def _build_export_a_shared_flow_argument(self):
        pass

    def _build_get_a_shared_flow_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "get",
            aliases=["get-a-shared-flow"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Gets a shared flow by name, including a list of its revisions.",
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org).get_a_shared_flow(args.name).text
            )
        )

    def _build_deploy_a_shared_flow_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "deploy",
            aliases=["deploy-a-shared-flow"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                # self._file_parser(),
                self._environment_parser(),
            ],
            help="Deploys a shared flow revision to an environment in an organization. Shared flows cannot be used until they have been deployed to an environment. If you experience HTTP 500 errors during deployment, consider using the override parameter to deploy the shared flow in place of a revision currently deployed. The size limit of a shared flow bundle is 15 MB. WARNING: currently, --override does not seem to work on Apigee Edge. To counter this, existing shared flow revisions in the target environment will be undeployed before deploying a new revision, resulting in some downtime.",
        )
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            "-f",
            "--file",
            action="store",
            help="file path of the shared flow configuration in ZIP format",
            # required=True,
            type=ArgumentType.isfile,
            default=None,
        )
        group.add_argument(
            "-r",
            "--revision-number",
            type=int,
            help="revision number",
            # required=True,
            default=None,
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.add_argument(
            "--override",
            action="store_true",
            help='Set this flag to "true" to deploy the shared flow revision in place of the revision currently deployed (also known as "zero downtime" deployment).',
        )
        parser.add_argument(
            "--delay",
            type=int,
            default=0,
            help="Enforces a delay, measured in seconds, before the currently deployed API proxy revision is undeployed and replaced by the new revision that is being deployed. Use this setting to minimize the impact of deployment on in-flight transactions. The default value is 0.",
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org)
                .deploy_a_shared_flow(
                    args.environment,
                    args.name,
                    args.revision_number,
                    override=args.override,
                    delay=args.delay,
                    shared_flow_file=args.file,
                )
                .text
            )
        )

    def _build_undeploy_a_shared_flow_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "undeploy",
            aliases=["undeploy-a-shared-flow"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Undeploys a shared flow revision from an environment.",
        )
        parser.add_argument(
            "-r",
            "--revision-number",
            type=int,
            help="revision number",
            required=True,
            default=None,
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org)
                .undeploy_a_shared_flow(
                    args.environment, args.name, args.revision_number,
                )
                .text
            )
        )

    def _build_get_deployment_environments_for_shared_flows_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "get-deployment-environments",
            aliases=["get-deployment-environments-for-shared-flows"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Gets an array of the environments to which the shared flow is deployed.",
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.add_argument(
            "-r",
            "--revision-number",
            type=int,
            help="revision number",
            required=True,
            default=None,
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org)
                .get_deployment_environments_for_shared_flows(
                    args.name, args.revision_number
                )
                .text
            )
        )

    def _build_delete_a_shared_flow_argument(self):
        pass

    def _build_attach_a_shared_flow_to_a_flow_hook_argument(self):
        pass

    def _build_detaches_a_shared_flow_from_a_flow_hook_argument(self):
        pass

    def _build_get_the_shared_flow_attached_to_a_flow_hook_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "get-flow-hook",
            aliases=["get-the-shared-flow-attached-to-a-flow-hook"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Returns the name of the shared flow attached to the specified flow hook. If there's no shared flow attached to the flow hook, the API does not return an error; it simply does not return a name in the response. Specify one of these flowhook locations in the API: PreProxyFlowHook, PreTargetFlowHook, PostTargetFlowHook, PostProxyFlowHook",
        )
        parser.add_argument(
            "--flow-hook",
            help="flow hook to which the shared flow is attached",
            required=True,
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org)
                .get_the_shared_flow_attached_to_a_flow_hook(
                    args.environment, args.flow_hook
                )
                .text
            )
        )

    def _build_get_shared_flow_deployments_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "get-deployments",
            aliases=["get-shared-flow-deployments"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="View all shared flow deployments.",
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org).get_shared_flow_deployments(args.name).text
            )
        )

    def _build_get_shared_flow_revisions_argument(self):
        parser = self._parser_sharedflows.add_parser(
            "get-revisions",
            aliases=["get-shared-flow-revisions"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="List the revisions of a shared flow.",
        )
        parser.add_argument("-n", "--name", help="shared flow name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Sharedflows(args, args.org).get_shared_flow_revisions(args.name).text
            )
        )

    def _create_parser(self):
        self._build_get_a_list_of_shared_flows_argument()
        self._build_import_a_shared_flow_argument()
        self._build_export_a_shared_flow_argument()
        self._build_get_a_shared_flow_argument()
        self._build_deploy_a_shared_flow_argument()
        self._build_undeploy_a_shared_flow_argument()
        self._build_get_deployment_environments_for_shared_flows_argument()
        self._build_delete_a_shared_flow_argument()
        self._build_attach_a_shared_flow_to_a_flow_hook_argument()
        self._build_detaches_a_shared_flow_from_a_flow_hook_argument()
        self._build_get_the_shared_flow_attached_to_a_flow_hook_argument()
        self._build_get_shared_flow_deployments_argument()
        self._build_get_shared_flow_revisions_argument()



import argparse

from apigee.api.permissions import Permissions
from apigee.parsers.file_parser import FileParser
from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserPermissions:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_permissions = self._parser.add_parser(
            "permissions", aliases=["perms"], help="manage permissions for a role"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_permissions(self):
        return self._parser_permissions

    @parser_permissions.setter
    def parser_permissions(self, value):
        self._parser_permissions = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    def __call__(self):
        return self._parser

    def _build_create_permissions_argument(self):
        create_permissions = self._parser_permissions.add_parser(
            "create",
            aliases=["create-permissions"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Create permissions for a role.",
        )
        create_permissions.add_argument("-n", "--name", help="name", required=True)
        create_permissions.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_permissions.set_defaults(
            func=lambda args: console.log(
                Permissions(args, args.org, args.name)
                .create_permissions(args.body)
                .text
            )
        )

    def _build_team_permissions_argument(self):
        team_permissions = self._parser_permissions.add_parser(
            "template",
            aliases=["template-permissions", "team-permissions", "team"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._file_parser(),
            ],
            help="Create permissions for a role using a template file.",
        )
        team_permissions.add_argument(
            "-n", "--name", help="name of user role", required=True
        )
        team_permissions.add_argument(
            "--placeholder-key",
            help="placeholder key to replace with a placeholder value",
            required=False,
        )
        team_permissions.add_argument(
            "--placeholder-value",
            help="placeholder value to replace placeholder key. default is an empty string.",
            default="",
            required=False,
        )
        team_permissions.set_defaults(
            func=lambda args: console.log(
                Permissions(args, args.org, args.name)
                .team_permissions(
                    args.file,
                    placeholder_key=args.placeholder_key,
                    placeholder_value=args.placeholder_value,
                )
                .text
            )
        )

    def _build_get_permissions_argument(self):
        get_permissions = self._parser_permissions.add_parser(
            "get",
            aliases=["get-permissions"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Get permissions for a role.",
        )
        get_permissions.add_argument("-n", "--name", help="name", required=True)
        get_permissions.add_argument(
            "-j",
            "--json",
            action="store_const",
            const="text",
            help="use json output",
            default="table",
        )
        # get_permissions.add_argument('--max-colwidth', help='max column width', type=int, default=40)
        get_permissions.add_argument(
            "--showindex", action="store_true", help="add row indices"
        )
        get_permissions.add_argument(
            "--tablefmt",
            help="defines how the table is formatted",
            default="plain",
            choices=(
                "plain",
                "simple",
                "github",
                "grid",
                "fancy_grid",
                "pipe",
                "orgtbl",
                "jira",
                "presto",
                "psql",
                "rst",
                "mediawiki",
                "moinmoin",
                "youtrack",
                "html",
                "latex",
                "latex_raw",
                "latex_booktabs",
                "textile",
            ),
            type=str,
        )
        get_permissions.set_defaults(
            func=lambda args: console.log(
                Permissions(args, args.org, args.name).get_permissions(
                    formatted=True,
                    format=args.json,
                    showindex=args.showindex,
                    tablefmt=args.tablefmt,
                )
            )
        )

    def _create_parser(self):
        self._build_create_permissions_argument()
        self._build_team_permissions_argument()
        self._build_get_permissions_argument()



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/deployments

API Platform Base Path: https://api.enterprise.apigee.com/v1/o/{org_name}

API Resource Path: /apis/{api_name}/deployments

API proxies that are actively deployed in environments on Apigee Edge.
"""

import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.deployments import IDeployments, DeploymentsSerializer
from apigee.util import authorization


class Deployments(IDeployments):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_api_proxy_deployment_details(
        self,
        formatted=False,
        format="text",
        showindex=False,
        tablefmt="plain",
        revision_name_only=False,
    ):
        """Gets details for a specific API proxy deployed in a given environment

        Args:
            formatted (bool, optional): If True, format ``requests.Response()``.
                Defaults to False.
            format (str, optional): Specify how to format response.
                Defaults to 'text'.
            showindex (bool, optional): If True, show table index.
                Defaults to False.
            tablefmt (str, optional): Specify table format. Defaults to 'plain'.
            revision_name_only (bool, optional): If True, truncate response
                information.
                Defaults to False.

        Returns:
            requests.Response(): Response if ``formatted`` is False,
            else return a ``formatted`` value.
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apis/{self._api_name}/deployments"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        if formatted:
            if revision_name_only:
                # return DeploymentsSerializer().serialize_details(resp, 'table', max_colwidth=args.max_colwidth)
                return DeploymentsSerializer().serialize_details(
                    resp, format, showindex=showindex, tablefmt=tablefmt
                )
            return DeploymentsSerializer().serialize_details(resp, "text")
        return resp



import argparse

from apigee.api.maskconfigs import Maskconfigs

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserMaskconfigs:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_maskconfigs = self._parser.add_parser(
            "maskconfigs", aliases=["masks"], help="manage data masks"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_maskconfigs(self):
        return self._parser_maskconfigs

    @parser_maskconfigs.setter
    def parser_maskconfigs(self, value):
        self._parser_maskconfigs = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    def __call__(self):
        return self._parser

    def _build_create_data_masks_for_an_api_proxy_argument(self):
        create_data_masks_for_an_api_proxy = self._parser_maskconfigs.add_parser(
            "create-api",
            aliases=["create-data-masks-for-an-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Create a data mask for an API proxy. You can capture message content to assist in runtime debugging of APIs calls. In many cases, API traffic contains sensitive data, such credit cards or personally identifiable health information (PHI) that needs to filtered out of the captured message content. Data masks enable you to specify data that will be filtered out of trace sessions. Data masking is only enabled when a trace session (also called a 'debug' session) is enabled for an API proxy. If no trace session are enabled on an API proxy, then the data will not be masked.",
        )
        create_data_masks_for_an_api_proxy.add_argument(
            "-n", "--name", help="name", required=True
        )
        create_data_masks_for_an_api_proxy.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_data_masks_for_an_api_proxy.set_defaults(
            func=lambda args: console.log(
                Maskconfigs(args, args.org, args.name)
                .create_data_masks_for_an_api_proxy(args.body)
                .text
            )
        )

    def _build_delete_data_masks_for_an_api_proxy_argument(self):
        delete_data_masks_for_an_api_proxy = self._parser_maskconfigs.add_parser(
            "delete-api",
            aliases=["delete-data-masks-for-an-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Delete a data mask for an API proxy.",
        )
        delete_data_masks_for_an_api_proxy.add_argument(
            "-n", "--name", help="name", required=True
        )
        delete_data_masks_for_an_api_proxy.add_argument(
            "--maskconfig-name", help="data mask name", required=True
        )
        delete_data_masks_for_an_api_proxy.set_defaults(
            func=lambda args: console.log(
                Maskconfigs(args, args.org, args.name)
                .delete_data_masks_for_an_api_proxy(args.maskconfig_name)
                .text
            )
        )

    def _build_get_data_mask_details_for_an_api_proxy_argument(self):
        get_data_mask_details_for_an_api_proxy = self._parser_maskconfigs.add_parser(
            "get-api",
            aliases=["get-data-mask-details-for-an-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Get the details for a data mask for an API proxy.",
        )
        get_data_mask_details_for_an_api_proxy.add_argument(
            "-n", "--name", help="name", required=True
        )
        get_data_mask_details_for_an_api_proxy.add_argument(
            "--maskconfig-name", help="data mask name", required=True
        )
        get_data_mask_details_for_an_api_proxy.set_defaults(
            func=lambda args: console.log(
                Maskconfigs(args, args.org, args.name)
                .get_data_mask_details_for_an_api_proxy(args.maskconfig_name)
                .text
            )
        )

    def _build_list_data_masks_for_an_api_proxy_argument(self):
        list_data_masks_for_an_api_proxy = self._parser_maskconfigs.add_parser(
            "list-api",
            aliases=["list-data-masks-for-an-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="List all data masks for an API proxy.",
        )
        list_data_masks_for_an_api_proxy.add_argument(
            "-n", "--name", help="name", required=True
        )
        list_data_masks_for_an_api_proxy.set_defaults(
            func=lambda args: console.log(
                Maskconfigs(args, args.org, args.name)
                .list_data_masks_for_an_api_proxy()
                .text
            )
        )

    def _build_list_data_masks_for_an_organization_argument(self):
        list_data_masks_for_an_organization = self._parser_maskconfigs.add_parser(
            "list",
            aliases=["list-data-masks-for-an-organization"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="List all data masks for an organization.",
        )
        list_data_masks_for_an_organization.set_defaults(
            func=lambda args: console.log(
                Maskconfigs(args, args.org, None)
                .list_data_masks_for_an_organization()
                .text
            )
        )

    def _build_push_data_masks_for_an_api_proxy_argument(self):
        push_data_masks_for_an_api_proxy = self._parser_maskconfigs.add_parser(
            "push",
            aliases=["push-data-masks-for-an-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._file_parser(),
            ],
            help="Push data mask for an API proxy to Apigee. This will create/update a data mask.",
        )
        push_data_masks_for_an_api_proxy.add_argument(
            "-n", "--name", help="name", required=True
        )
        push_data_masks_for_an_api_proxy.set_defaults(
            func=lambda args: Maskconfigs(
                args, args.org, args.name
            ).push_data_masks_for_an_api_proxy(args.file)
        )

    def _create_parser(self):
        self._build_create_data_masks_for_an_api_proxy_argument()
        self._build_delete_data_masks_for_an_api_proxy_argument()
        self._build_get_data_mask_details_for_an_api_proxy_argument()
        self._build_list_data_masks_for_an_api_proxy_argument()
        self._build_list_data_masks_for_an_organization_argument()
        self._build_push_data_masks_for_an_api_proxy_argument()



import argparse

from apigee.api.apiproducts import Apiproducts

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserApiproducts:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_apiproducts = self._parser.add_parser(
            "products", aliases=["prods"], help="manage API products"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_apiproducts(self):
        return self._parser_apiproducts

    @parser_apiproducts.setter
    def parser_apiproducts(self, value):
        self._parser_apiproducts = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_create_api_product_argument(self):
        create_api_product = self._parser_apiproducts.add_parser(
            "create",
            aliases=["create-api-product"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Creates an API product in an organization.",
        )
        create_api_product.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_api_product.set_defaults(
            func=lambda args: console.log(
                Apiproducts(args, args.org, args.name)
                .create_api_product(args.body)
                .text
            )
        )

    def _build_delete_api_product_argument(self):
        delete_api_product = self._parser_apiproducts.add_parser(
            "delete",
            aliases=["delete-api-product"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Deletes an API product from an organization.",
        )
        delete_api_product.add_argument("-n", "--name", help="name", required=True)
        delete_api_product.set_defaults(
            func=lambda args: console.log(
                Apiproducts(args, args.org, args.name).delete_api_product().text
            )
        )

    def _build_list_api_products_argument(self):
        list_api_products = self._parser_apiproducts.add_parser(
            "list",
            aliases=["list-api-products"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Get a list of all API product names for an organization.",
        )
        list_api_products.add_argument(
            "--expand",
            action="store_true",
            help="Set to 'true' to get expanded details about each product.",
        )
        list_api_products.add_argument(
            "--count",
            default=1000,
            type=int,
            help="Number of API products to return in the API call. The maximum limit is 1000. Use with the startkey to provide more targeted filtering.",
        )
        list_api_products.add_argument(
            "--startkey",
            default="",
            help="Returns a list of API products starting with the specified API product.",
        )
        list_api_products.set_defaults(
            func=lambda args: console.log(
                Apiproducts(args, args.org, None).list_api_products(
                    prefix=args.prefix,
                    expand=args.expand,
                    count=args.count,
                    startkey=args.startkey,
                )
            )
        )

    def _build_get_api_product_argument(self):
        get_api_product = self._parser_apiproducts.add_parser(
            "get",
            aliases=["get-api-product"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help='Gets configuration data for an API product. The API product name required in the request URL is not the "Display Name" value displayed for the API product in the Edge UI. While they may be the same, they are not always the same depending on whether the API product was created via UI or API.',
        )
        get_api_product.add_argument("-n", "--name", help="name", required=True)
        get_api_product.set_defaults(
            func=lambda args: console.log(
                Apiproducts(args, args.org, args.name).get_api_product().text
            )
        )

    def _build_update_api_product_argument(self):
        update_api_product = self._parser_apiproducts.add_parser(
            "update",
            aliases=["update-api-product"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Updates an existing API product. You must include all required values, whether or not you are updating them, as well as any optional values that you are updating.",
        )
        update_api_product.add_argument("-n", "--name", help="name", required=True)
        update_api_product.add_argument(
            "-b", "--body", help="request body", required=True
        )
        update_api_product.set_defaults(
            func=lambda args: console.log(
                Apiproducts(args, args.org, args.name)
                .update_api_product(args.body)
                .text
            )
        )

    def _build_push_apiproducts_argument(self):
        push_apiproducts = self._parser_apiproducts.add_parser(
            "push",
            aliases=["push-apiproducts"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._file_parser(),
            ],
            help="Push API product to Apigee. This will create/update an API product.",
        )
        push_apiproducts.set_defaults(
            func=lambda args: Apiproducts(args, args.org, None).push_apiproducts(
                args.file
            )
        )

    def _create_parser(self):
        self._build_create_api_product_argument()
        self._build_delete_api_product_argument()
        self._build_list_api_products_argument()
        self._build_get_api_product_argument()
        self._build_update_api_product_argument()
        self._build_push_apiproducts_argument()



#!/usr/bin/env python
"""https://apidocs.apigee.com/api-reference/content/shared-flows-and-flow-hooks-management-api"""

import json
import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.sharedflows import ISharedflows, SharedflowsSerializer
from apigee.util import authorization, console


class Sharedflows(ISharedflows):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_a_list_of_shared_flows(self, prefix=None):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return SharedflowsSerializer().serialize_details(resp, "json", prefix=prefix)

    def import_a_shared_flow(self, shared_flow_file, shared_flow_name):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows?action=import&name={shared_flow_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "multipart/form-data"},
            self._auth,
        )
        with open(shared_flow_file, "rb") as f:
            resp = requests.post(
                uri, headers=hdrs, files={"file": ("sharedflow.zip", f)}
            )
        resp.raise_for_status()
        return resp

    # def export_a_shared_flow(self, shared_flow_name, revision_number):
    #     pass

    def get_a_shared_flow(self, shared_flow_name):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows/{shared_flow_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def deploy_a_shared_flow(
        self,
        environment,
        shared_flow_name,
        revision_number,
        override=False,
        delay=0,
        shared_flow_file=None,
    ):
        # we need to undeploy existing deployed revisions in the target environment first,
        # otherwise each revision will remain deployed (will not undeploy with each new revision)
        self.undeploy_shared_flow_revisions_in_environment(
            environment, shared_flow_name
        )
        if shared_flow_file:
            revision_number = int(
                self.import_a_shared_flow(shared_flow_file, shared_flow_name).json()[
                    "revision"
                ]
            )
        # the `override` query parameter does not seem to work on Apigee Edge,
        # therefore we need to run undeploy_shared_flow_revisions_in_environment() first (as above)
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/sharedflows/{shared_flow_name}/revisions/{revision_number}/deployments?override={'true' if override else 'false'}&delay={delay}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth,)
        resp = requests.post(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def undeploy_shared_flow_revisions_in_environment(
        self, environment, shared_flow_name
    ):
        resp = self.get_shared_flow_deployments(shared_flow_name)
        for deployment in resp.json()["environment"]:
            if deployment["name"] == environment:
                for detail in deployment["revision"]:
                    revision_number = int(detail["name"])
                    self.undeploy_a_shared_flow(
                        environment, shared_flow_name, revision_number
                    )
        return resp

    def undeploy_a_shared_flow(self, environment, shared_flow_name, revision_number):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/sharedflows/{shared_flow_name}/revisions/{revision_number}/deployments"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_deployment_environments_for_shared_flows(
        self, shared_flow_name, revision_number
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows/{shared_flow_name}/revisions/{revision_number}/deployments"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_a_shared_flow(self, shared_flow_name):
        pass

    # def attach_a_shared_flow_to_a_flow_hook(self, environment, flow_hook, request_body):
    #     pass
    #
    # def detaches_a_shared_flow_from_a_flow_hook(self, environment, flow_hook):
    #     pass

    def get_the_shared_flow_attached_to_a_flow_hook(self, environment, flow_hook):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/flowhooks/{flow_hook}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_shared_flow_deployments(self, shared_flow_name):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows/{shared_flow_name}/deployments"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_shared_flow_revisions(self, shared_flow_name):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/sharedflows/{shared_flow_name}/revisions"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp



#!/usr/bin/env python
"""https://apidocs.apigee.com/api-reference/content/shared-flows-and-flow-hooks-management-api"""

import json
from abc import ABC, abstractmethod


class ISharedflows:
    def __init__(self, auth, org_name):
        self._auth = auth
        self._org_name = org_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @abstractmethod
    def get_a_list_of_shared_flows(self, prefix=None):
        pass

    @abstractmethod
    def import_a_shared_flow(self, shared_flow_file, shared_flow_name):
        pass

    @abstractmethod
    def export_a_shared_flow(self, shared_flow_name, revision_number):
        pass

    @abstractmethod
    def get_a_shared_flow(self, shared_flow_name):
        pass

    @abstractmethod
    def deploy_a_shared_flow(
        self,
        environment,
        shared_flow_name,
        revision_number,
        override=False,
        delay=0,
        shared_flow_file=None,
    ):
        pass

    @abstractmethod
    def undeploy_a_shared_flow(self, environment, shared_flow_name, revision_number):
        pass

    @abstractmethod
    def get_deployment_environments_for_shared_flows(
        self, shared_flow_name, revision_number
    ):
        pass

    @abstractmethod
    def delete_a_shared_flow(self, shared_flow_name):
        pass

    @abstractmethod
    def attach_a_shared_flow_to_a_flow_hook(self, environment, flow_hook, request_body):
        pass

    @abstractmethod
    def detaches_a_shared_flow_from_a_flow_hook(self, environment, flow_hook):
        pass

    @abstractmethod
    def get_the_shared_flow_attached_to_a_flow_hook(self, environment, flow_hook):
        pass

    @abstractmethod
    def get_shared_flow_deployments(self, shared_flow_name):
        pass

    @abstractmethod
    def get_shared_flow_revisions(self, shared_flow_name):
        pass


class SharedflowsSerializer:
    def serialize_details(self, sharedflows, format, prefix=None):
        resp = sharedflows
        if format == "text":
            return sharedflows.text
        sharedflows = sharedflows.json()
        if prefix:
            sharedflows = [
                sharedflow
                for sharedflow in sharedflows
                if sharedflow.startswith(prefix)
            ]
        if format == "json":
            return json.dumps(sharedflows)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import json
import os
import re
import sys
import zipfile


def makedirs(path):
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except FileExistsError:
            pass


def path_exists(file):
    if os.path.exists(file):
        sys.exit(f"error: {os.path.abspath(file)} already exists")


def paths_exist(files):
    for file in files:
        path_exists(file)


def extractzip(source, dest):
    with zipfile.ZipFile(source, "r") as zip_ref:
        zip_ref.extractall(dest)


def writezip(file, content):
    with open(file, "wb") as f:
        f.write(content)


# def serializepath(path_items, separator='/'):
#     pass


def splitpath(path, delimiter="[/\\\\]"):
    return re.split(delimiter, path)


def touch(path):
    try:
        with open(path, "x"):
            os.utime(path, None)
    except FileNotFoundError:
        os.makedirs(os.path.split(path)[0])
    except FileExistsError:
        pass


def isfile(f):
    return os.path.isfile(f)


def isdir(d):
    return os.path.isdir(d)


def read_file(file, type="text"):
    with open(file, "r") as f:
        if type == "json":
            return json.loads(f.read())
        return f.read()


def write_file(content, path, fs_write=True):
    if not fs_write:
        return
    touch(path)
    with open(path, "w") as f:
        try:
            f.write(content)
        except TypeError:
            if isinstance(content, dict) or isinstance(content, list):
                f.write(json.dumps(content))



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api-reference/content/data-masks

Base Path: https://api.enterprise.apigee.com/v1/o/{org_name}

API Resource Path: /maskconfigs

Specify data that will be filtered out of trace sessions

Edge enables developers to capture message content to enable runtime debugging
of APIs calls.
In many cases, API traffic contains sensitive data, such credit cards or
personally identifiable health information (PHI)
that needs to filtered out of the captured message content.
Mask configurations enable you to specify data that will be filtered out of
trace sessions.
Masking configurations can be set globally (at the organization-level) or
locally (at the API proxy level).
"""

import json
import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.maskconfigs import IMaskconfigs
from apigee.util import authorization, console


class Maskconfigs(IMaskconfigs):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_data_masks_for_an_api_proxy(self, request_body):
        """Create a data mask for an API proxy.

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apis/{self._api_name}/maskconfigs"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_data_masks_for_an_api_proxy(self, maskconfig_name):
        """Delete a data mask for an API proxy.

        Args:
            maskconfig_name (str): The data mask to delete.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apis/{self._api_name}/maskconfigs/{maskconfig_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_data_mask_details_for_an_api_proxy(self, maskconfig_name):
        """Get the details for a data mask for an API proxy.

        Args:
            maskconfig_name (str): The data mask name.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apis/{self._api_name}/maskconfigs/{maskconfig_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_data_masks_for_an_api_proxy(self):
        """List all data masks for an API proxy.

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/apis/{self._api_name}/maskconfigs"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_data_masks_for_an_organization(self):
        """List all data masks for an organization.

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/maskconfigs"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def push_data_masks_for_an_api_proxy(self, file):
        """Push data mask file to Apigee

        This will create a data mask if it does not exist and update if it does.

        Args:
            environment (str): Apigee environment.
            file (str): The file path.

        Returns:
            None

        Raises:
            HTTPError: If response status code is not successful or 404.
        """
        with open(file) as f:
            body = f.read()
        maskconfig = json.loads(body)
        maskconfig_name = maskconfig["name"]
        try:
            self.get_data_mask_details_for_an_api_proxy(maskconfig_name)
            console.log("Updating", maskconfig_name, "for", self._api_name)
            console.log(self.create_data_masks_for_an_api_proxy(body).text)
        except HTTPError as e:
            if e.response.status_code not in [404]:
                raise e
            console.log("Creating", maskconfig_name, "for", self._api_name)
            console.log(self.create_data_masks_for_an_api_proxy(body).text)



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/deployments"""

import json
from abc import ABC, abstractmethod

# import pandas as pd
# from pandas.io.json import json_normalize
from tabulate import tabulate


class IDeployments:
    def __init__(self, auth, org_name, api_name):
        self._auth = auth
        self._org_name = org_name
        self._api_name = api_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def api_name(self):
        return self._api_name

    @api_name.setter
    def api_name(self, value):
        self._api_name = value

    @abstractmethod
    def get_api_proxy_deployment_details(
        self,
        formatted=False,
        format="text",
        showindex=False,
        tablefmt="plain",
        revision_name_only=False,
    ):
        pass


class DeploymentsSerializer:
    def serialize_details(
        self, deployment_details, format, showindex=False, tablefmt="plain"
    ):
        if format == "text":
            return deployment_details.text
        revisions = []
        for i in deployment_details.json()["environment"]:
            revisions.append(
                {"name": i["name"], "revision": [j["name"] for j in i["revision"]]}
            )
        if format == "json":
            return json.dumps(revisions)
        elif format == "table":  # max_colwidth=40
            # pd.set_option('display.max_colwidth', max_colwidth)
            # return pd.DataFrame.from_dict(json_normalize(revisions), orient='columns')
            table = [[rev["name"], rev["revision"]] for rev in revisions]
            headers = []
            if showindex == "always" or showindex is True:
                headers = ["id", "name", "revision"]
            elif showindex == "never" or showindex is False:
                headers = ["name", "revision"]
            return tabulate(table, headers, showindex=showindex, tablefmt=tablefmt)
        # else:
        #     raise ValueError(format)
        return deployment_details



import argparse

from apigee.util import configure


class ParserConfigure:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    def __call__(self):
        return self._parser

    def _build_parser_configure_argument(self):
        parser_configure = self._parser.add_parser(
            "configure", help="configure credentials"
        )
        parser_configure.add_argument(
            "-P", "--profile", help="name of profile to create", default="default"
        )
        parser_configure.set_defaults(
            func=lambda args: configure.Configure(args).__call__()
        )

    def _create_parser(self):
        self._build_parser_configure_argument()



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/api_resources/51-targetservers

TargetServers are used to decouple TargetEndpoint HTTPTargetConnections from
concrete URLs for backend services.

To do so, an HTTPConnection can be configured to use a LoadBalancer that lists
one or more 'named' TargetSevers.
Using TargetServers, you can create an HTTPTargetConnection that calls a
different backend server based on the environment where the API proxy is
deployed.
"""

import json
import requests
from requests.exceptions import HTTPError

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.targetservers import ITargetservers, TargetserversSerializer
from apigee.util import authorization, console


class Targetservers(ITargetservers):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_a_targetserver(self, environment, request_body):
        """Create a TargetServer in the specified environment.

        Args:
            environment (str): Apigee environment.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/targetservers"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_a_targetserver(self, environment):
        """Delete a TargetServer configuration from an environment.

        Args:
            environment (str): Apigee environment.

        Returns:
            requests.Response(): Information about the deleted TargetServer.
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/targetservers/{self._targetserver_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_targetservers_in_an_environment(self, environment, prefix=None):
        """List all TargetServers in an environment.

        Args:
            environment (str): Apigee environment.
            prefix (str, optional): Filter results by a prefix string.
                Defaults to None.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/targetservers"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return TargetserversSerializer().serialize_details(resp, "json", prefix=prefix)

    def get_targetserver(self, environment):
        """Returns a TargetServer definition.

        Args:
            environment (str): Apigee environment.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/targetservers/{self._targetserver_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def update_a_targetserver(self, environment, request_body):
        """Modifies an existing TargetServer.

        Note: In the request body when Content-Type is set to application/json,
        use the element sSLInfo in the request body, not SSLInfo (note the
        case).
        You must use the case sSLInfo exactly.
        Using SSLInfo will cause an error when Content-Type is set to
        application/json.
        The case does not matter if Content-Type is application/xml.

        Args:
            environment (str): Apigee environment.
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/targetservers/{self._targetserver_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.put(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def push_targetserver(self, environment, file):
        """Push TargetSever file to Apigee

        This will create a TargetSever if it does not exist and update if it
        does.

        Args:
            environment (str): Apigee environment.
            file (str): The file path.

        Returns:
            None

        Raises:
            HTTPError: If response status code is not successful or 404.
        """
        with open(file) as f:
            body = f.read()
        targetserver = json.loads(body)
        self._targetserver_name = targetserver["name"]
        try:
            self.get_targetserver(environment)
            console.log("Updating", self._targetserver_name)
            console.log(self.update_a_targetserver(environment, body).text)
        except HTTPError as e:
            if e.response.status_code not in [404]:
                raise e
            console.log("Creating", self._targetserver_name)
            console.log(self.create_a_targetserver(environment, body).text)



import builtins
import sys

import apigee


def log(*message, status=None, silent=False, curr_verbosity=0, expc_verbosity=0):
    toggle_silent = False
    toggle_verbose = 0
    try:  # hack
        toggle_silent = builtins.APIGEE_CLI_TOGGLE_SILENT
        toggle_verbose = builtins.APIGEE_CLI_TOGGLE_VERBOSE
    except AttributeError:
        pass
    if silent or toggle_silent:
        if status:
            sys.exit(status)
        return
    if curr_verbosity or toggle_verbose >= expc_verbosity:
        print(*message)
    if status:
        sys.exit(status)



import argparse

from apigee import APIGEE_CLI_PREFIX
from apigee.util import authorization, console


class PrefixParser:
    def __init__(self, **kwargs):
        self._parent_parser = argparse.ArgumentParser(add_help=False)
        # self._build_profile_argument()
        # self._profile = self._parent_parser.parse_known_args()[0].profile
        self._profile = kwargs.get("profile", "default")
        self._prefix = authorization.get_credential(self._profile, "prefix")
        self._create_parser()

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def profile(self):
        return self._profile

    @profile.setter
    def profile(self, value):
        self._profile = value

    def __call__(self):
        return self._parent_parser

    # def _build_profile_argument(self):
    #     self._parent_parser.add_argument('-P', '--profile', action='store', help='name of credentials profile', default='default')

    def _build_prefix_argument(self):
        self._parent_parser.add_argument(
            "--prefix",
            help="prefix filter for apigee items",
            default=APIGEE_CLI_PREFIX if self._prefix is None else self._prefix,
        )

    def _create_parser(self):
        self._build_prefix_argument()



import argparse

from apigee.api.stats import Stats

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserStats:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_stats = self._parser.add_parser(
            "stats", help="access metrics collected by Apigee Edge"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_stats(self):
        return self._parser_stats

    @parser_stats.setter
    def parser_stats(self, value):
        self._parser_stats = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_get_license_utilization_argument(self):
        parser = self._parser_stats.add_parser(
            "get-license-utilization",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Gets current license utilization per environment.",
        )
        parser.add_argument(
            "-e",
            "--environments",
            help="list of environments",
            nargs="+",
            required=True,
        )
        parser.add_argument(
            "--time-range",
            help="time range e.g. '01/01/2020 00:00~04/01/2020 00:00'",
            required=True,
        )
        parser.add_argument("--tzo", help="tzo", type=int, default=None)
        parser.add_argument(
            "--api-calls-per-year",
            help="Apigee license per year (API Calls). Default is 10,000,000,000.",
            type=int,
            default=10000000000,
        )
        parser.set_defaults(
            func=lambda args: Stats(args, args.org).get_license_utilization(
                args.time_range,
                environments=args.environments,
                tzo=args.tzo,
                api_calls_per_year=args.api_calls_per_year,
            )
        )

    def _create_parser(self):
        self._build_get_license_utilization_argument()



#!/usr/bin/env python

import configparser
import os
from pathlib import Path

from apigee import APIGEE_CLI_DIRECTORY
from apigee import APIGEE_CLI_CREDENTIALS_FILE
from apigee.util.os import makedirs


class Configure:
    def __init__(self, args):
        self._args = args
        self._profile = args.profile
        self._config = configparser.ConfigParser()
        self._config.read(APIGEE_CLI_CREDENTIALS_FILE)
        self.KEY_LIST = ("username", "password", "mfa_secret", "org", "prefix")
        try:
            self._profile_dict = dict(self._config._sections[self._profile])
            for key in self.KEY_LIST:
                if key not in self._profile_dict:
                    self._profile_dict[key] = None
        except KeyError:
            self._profile_dict = {k: None for k in self.KEY_LIST}

    def __call__(self):
        self._main()

    def _mask_secret(self, secret):
        return "*" * 16 if secret else secret

    def _remove_empty_keys(self, dict):
        return {k: v for k, v in dict.items() if v}

    def _save_config(self, file):
        makedirs(APIGEE_CLI_DIRECTORY)
        with open(file, "w") as configfile:
            self._config.write(configfile)

    def _main(self):
        self._profile_dict["username"] = input(
            f"Apigee username (email) [{self._profile_dict['username']}]: "
        )
        self._profile_dict["password"] = input(
            f"Apigee password [{self._mask_secret(self._profile_dict['password'])}]: "
        )
        self._profile_dict["mfa_secret"] = input(
            f"Apigee MFA key (recommended) [{self._mask_secret(self._profile_dict['mfa_secret'])}]: "
        )
        self._profile_dict["org"] = input(
            f"Default Apigee organization (recommended) [{self._profile_dict['org']}]: "
        )
        self._profile_dict["prefix"] = input(
            f"Default team/resource prefix (recommended) [{self._profile_dict['prefix']}]: "
        )
        self._config[self._profile] = self._remove_empty_keys(self._profile_dict)
        self._save_config(APIGEE_CLI_CREDENTIALS_FILE)



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/debug-sessions"""

from abc import ABC, abstractmethod


class IDebugsessions:
    def __init__(self, auth, org_name):
        self._auth = auth
        self._org_name = org_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @abstractmethod
    def create_a_debug_session(
        self,
        environment,
        api_name,
        revision_number,
        session_name,
        timeout=600,
        filter="",
    ):
        pass

    @abstractmethod
    def delete_debug_session(
        self, environment, api_name, revision_number, session_name
    ):
        pass

    @abstractmethod
    def list_debug_sessions(self, environment, api_name, revision_number):
        pass

    @abstractmethod
    def get_debug_session_transaction_IDs(
        self, environment, api_name, revision_number, session_name
    ):
        pass

    @abstractmethod
    def get_debug_session_transaction_data(
        self, environment, api_name, revision_number, session_name, transaction_id
    ):
        pass


class DebugsessionsSerializer:
    def serialize_details(self, sessions, format, prefix=None):
        resp = sessions
        if format == "text":
            return sessions.text
        sessions = sessions.json()
        if prefix:
            sessions = [api for api in sessions if api.startswith(prefix)]
        if format == "json":
            return json.dumps(sessions)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



#!/usr/bin/env python
"""https://apidocs.apigee.com/api-reference/content/api-proxies"""

import json
import os
from abc import ABC, abstractmethod
from pathlib import Path

# from apigee.util.os import serializepath, splitpath


class IApis:
    def __init__(self, auth, org_name):
        self._auth = auth
        self._org_name = org_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @abstractmethod
    def delete_api_proxy_revision(self, api_name, revision_number):
        pass

    @abstractmethod
    def deploy_api_proxy_revision(
        self, api_name, environment, revision_number, delay=0, override=False
    ):
        pass

    @abstractmethod
    def delete_undeployed_revisions(self, api_name, save_last=0, dry_run=False):
        pass

    @abstractmethod
    def export_api_proxy(
        self, api_name, revision_number, fs_write=True, write=True, output_file=None
    ):
        pass

    @abstractmethod
    def get_api_proxy(self, api_name):
        pass

    @abstractmethod
    def list_api_proxies(self, prefix=None):
        pass

    @abstractmethod
    def list_api_proxy_revisions(self, api_name):
        pass

    @abstractmethod
    def undeploy_api_proxy_revision(self, api_name, environment, revision_number):
        pass

    @abstractmethod
    def force_undeploy_api_proxy_revision(self, api_name, environment, revision_number):
        pass


class ApisSerializer:
    def serialize_details(self, apis, format, prefix=None):
        resp = apis
        if format == "text":
            return apis.text
        apis = apis.json()
        if prefix:
            apis = [api for api in apis if api.startswith(prefix)]
        if format == "json":
            return json.dumps(apis)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp


class IPull:
    def __init__(self, auth, org_name, revision_number, environment, work_tree=None):
        self._auth = auth
        self._org_name = org_name
        if work_tree:
            if not os.path.exists(work_tree):
                os.makedirs(work_tree)
            self._work_tree = str(Path(work_tree).resolve())
        else:
            self._work_tree = os.getcwd()
        self._revision_number = revision_number
        self._environment = environment
        self._keyvaluemaps_dir = str(
            Path(self._work_tree) / "keyvaluemaps" / environment
        )
        self._targetservers_dir = str(
            Path(self._work_tree) / "targetservers" / environment
        )
        self._caches_dir = str(Path(self._work_tree) / "caches" / environment)
        # self._apiproxy_dir = str(Path(self._work_tree) / api_name)
        self._apiproxy_dir = str(Path(self._work_tree))
        self._zip_file = str(Path(self._apiproxy_dir).with_suffix(".zip"))

    def __call__(self, *args, **kwargs):
        self.pull(*args, **kwargs)

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def revision_number(self):
        return self._revision_number

    @revision_number.setter
    def revision_number(self, value):
        self._revision_number = value

    @property
    def environment(self):
        return self._environment

    @environment.setter
    def environment(self, value):
        self._environment = value

    # @property
    # def work_tree(self):
    #     return self._work_tree
    #
    # @work_tree.setter
    # def work_tree(self, value):
    #     self.__init__(self._args, self._api_name, self._revision_number, work_tree=value)

    @property
    def keyvaluemaps_dir(self):
        return self._keyvaluemaps_dir

    @keyvaluemaps_dir.setter
    def keyvaluemaps_dir(self, value):
        self._keyvaluemaps_dir = str(Path(self._work_tree) / value / environment)

    @property
    def targetservers_dir(self):
        return self._targetservers_dir

    @targetservers_dir.setter
    def targetservers_dir(self, value):
        self._targetservers_dir = str(Path(self._work_tree) / value / environment)

    @property
    def caches_dir(self):
        return self._caches_dir

    @caches_dir.setter
    def caches_dir(self, value):
        self._caches_dir = str(Path(self._work_tree) / value / environment)

    @property
    def apiproxy_dir(self):
        return self._apiproxy_dir

    @apiproxy_dir.setter
    def apiproxy_dir(self, value):
        self._apiproxy_dir = str(Path(self._work_tree) / value)

    @property
    def zip_file(self):
        return self._zip_file

    @zip_file.setter
    def zip_file(self, value):
        self._zip_file = str(Path(self._apiproxy_dir) / value)

    @abstractmethod
    def get_apiproxy_files(self, directory):
        pass

    @abstractmethod
    def get_keyvaluemap_dependencies(self, files):
        pass

    @abstractmethod
    def export_keyvaluemap_dependencies(self, environment, keyvaluemaps, force=False):
        pass

    @abstractmethod
    def get_targetserver_dependencies(self, files):
        pass

    @abstractmethod
    def export_targetserver_dependencies(
        self, environment, target_servers, force=False
    ):
        pass

    @abstractmethod
    def replace_substring(self, file, old, new):
        pass

    @abstractmethod
    def prefix_dependencies_in_work_tree(self, dependencies, prefix):
        pass

    @abstractmethod
    def get_apiproxy_basepath(self, directory):
        pass

    @abstractmethod
    def set_apiproxy_basepath(self, basepath, file):
        pass

    @abstractmethod
    def pull(self, api_name, dependencies=[], force=False, prefix=None, basepath=None):
        pass



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/debug-sessions

API Platform Base Path: https://api.enterprise.apigee.com/v1/o/{org_name}

API Resource Path: /debugsessions

A session configured in Apigee Edge to record specified messages and associated
pipeline processing metadata for debugging purposes
"""

import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.debugsessions import IDebugsessions
from apigee.util import authorization


class Debugsessions(IDebugsessions):
    def __init__(self, auth, org_name):
        self._auth = auth
        self._org_name = org_name

    def create_a_debug_session(
        self,
        environment,
        api_name,
        revision_number,
        session_name,
        timeout=600,
        filter="",
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/apis/{api_name}/revisions/{revision_number}/debugsessions?session={session_name}&timeout={timeout}"
        hdrs = authorization.set_header(
            {
                "Accept": "application/json",
                "Content-Type": "application/x-www-url-form-encoded",
            },
            self._auth,
        )
        if filter:
            uri += f"&{filter}"
        resp = requests.post(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_debug_session(
        self, environment, api_name, revision_number, session_name
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/apis/{api_name}/revisions/{revision_number}/debugsessions/{session_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_debug_sessions(self, environment, api_name, revision_number):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/apis/{api_name}/revisions/{revision_number}/debugsessions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_debug_session_transaction_IDs(
        self, environment, api_name, revision_number, session_name
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/apis/{api_name}/revisions/{revision_number}/debugsessions/{session_name}/data"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_debug_session_transaction_data(
        self, environment, api_name, revision_number, session_name, transaction_id
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/environments/{environment}/apis/{api_name}/revisions/{revision_number}/debugsessions/{session_name}/data/{transaction_id}"
        hdrs = authorization.set_header({"Accept": "application/xml"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/user-roles

API Platform Base Path: https://api.enterprise.apigee.com/v1/o/{org_name}

API Resource Path: /userroles

Roles for users in an organization on Apigee Edge.

User roles form the basis of role-based access in Apigee Edge.

Users are associated with one or more userroles. Each userrole defines a set of
permissions (GET, PUT, DELETE) on RBAC resources (defined by URI paths).

A userrole is scoped to an organization.
"""

import json
import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.userroles import IUserroles
from apigee.util import authorization


class Userroles(IUserroles):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def add_a_user_to_a_role(self, user_email):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/users?id={user_email}"
        hdrs = authorization.set_header(
            {
                "Accept": "application/json",
                "Content-Type": "application/x-www-form-urlencoded",
            },
            self._auth,
        )
        resp = requests.post(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def add_permissions_for_a_resource_to_a_user_role(self, request_body):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def add_permissions_for_multiple_resources_to_a_user_role(self, request_body):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/resourcepermissions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def create_a_user_role_in_an_organization(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        roles = []
        for role in self._role_name:
            roles.append({"name": role})
        body = {"role": roles}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_a_permission_for_a_resource(self, permission, resource_path):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions/{permission}?path={resource_path}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/octet-stream"},
            self._auth,
        )
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_resource_from_permissions(self, resource_path):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions?path={resource_path}&delete=true"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/octet-stream"},
            self._auth,
        )
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def delete_a_user_role(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}"
        hdrs = authorization.set_header(
            {
                "Accept": "application/json",
                "Content-Type": "application/x-www-form-urlencoded",
            },
            self._auth,
        )
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_a_role(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_resource_permissions_for_a_specific_role(self, resource_path=""):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions"
        if resource_path:
            uri += f"?path={resource_path}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/octet-stream"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_users_for_a_role(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/users"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_permissions_for_a_resource(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_user_roles(self):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def remove_user_membership_in_role(self, user_email):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/users/{user_email}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def verify_a_user_roles_permission_on_a_specific_RBAC_resource(
        self, permission, resource_path
    ):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/permissions/{permission}?path={resource_path}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/octet-stream"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def verify_user_role_membership(self, user_email):
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/userroles/{self._role_name}/users/{user_email}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp



#!/usr/bin/env python
"""Source: https://apidocs.apigee.com/api/developers-0

Base Path: https://api.enterprise.apigee.com/v1/o/{org_name}

API Resource Path: /developers

Developers implement client/consumer apps and must be registered with an
organization on Apigee Edge.
"""

import json
import requests

from apigee import APIGEE_ADMIN_API_URL
from apigee.abstract.api.developers import IDevelopers, DevelopersSerializer
from apigee.util import authorization


class Developers(IDevelopers):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def create_developer(
        self, first_name, last_name, user_name, attributes='{"attributes" : [ ]}'
    ):
        """Creates a profile for a developer in an organization.

        Once created, the developer can register an app and receive an API key.

        The developer is always created with a status of active.

        Args:
            first_name (str): The developer's first name.
            last_name (str): The developer's last name.
            user_name (str): The developer's user name.
            attributes (str, optional): JSON string.
                Defaults to '{"attributes" : [ ]}'.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = {
            "email": self._developer_email,
            "firstName": first_name,
            "lastName": last_name,
            "userName": user_name,
            "attributes": json.loads(attributes)["attributes"],
        }
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_developer(self):
        """Deletes a developer from an organization.

        All apps and API keys associated with the developer are also removed
        from the organization.
        All times in the response are UNIX times.

        To avoid permanently deleting developers and their artifacts, consider
        deactivating developers instead.

        Note the following:
          - With Apigee Edge for Public Cloud, deletion of the developer and
            associated artifacts happens asynchronously.
            The developer is deleted immediately, but the resources associated
            with that developer, such as apps, may take anywhere from a few
            seconds to a few minutes to be automatically deleted.
          - Apigee recommends that you use the developer's email address when
            calling this API.
            Developer IDs are generated internally by Apigee and are not
            guaranteed to stay the same over time.

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_developer(self):
        """Returns the profile for a developer by email address or ID.

        All time values are UNIX time values.

        The profile includes the developer's email address, ID, name, and other
        information.

        Note the following:
          - With Apigee Edge for Public Cloud, the response includes only the
            first 100 apps.

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_developer_by_app(self, app_name):
        """Gets the developer profile by app name.

        The profile retrieved is for the developer associated with the app in
        the organization.
        All time values are UNIX time values.

        Args:
            app_name (str): The name of the app.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers?app={app_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def list_developers(self, prefix=None, expand=False, count=100, startkey=""):
        """Lists all developers in an organization by email address.

        This call does not list any company developers who are a part of the
        designated organization.

        With Apigee Edge for Public Cloud:
          - A maximum of 1000 developers are returned per API call.
          - You can paginate the list of companies returned using the
            ``startKey`` and ``count`` query parameters.

        Args:
            prefix (str, optional): Filter results by a prefix string.
                Defaults to None.
            expand (bool, optional): If True, list developers exanded with
                details. Defaults to False.
            count (int, optional): Note: This parameter can be used with
                Apigee Edge for Public Cloud only.
                Limits the list to the number you specify.
                Use with the startKey parameter to provide more targeted
                filtering.
                The limit is 1000.
                Defaults to 100.
            startkey (str, optional): Note: This parameter can be used with
                Apigee Edge for Public Cloud only.
                To filter the keys that are returned, enter the email of a
                developer that the list will start with.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers?expand={expand}&count={count}&startKey={startkey}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return DevelopersSerializer().serialize_details(resp, "json", prefix=prefix)

    def set_developer_status(self, action):
        """Sets a developer's status to active or inactive for a specific
        organization.

        Run this API for each organization where you want to change the
        developer's status.

        By default, the status of a developer is set to active.
        Admins with proper permissions (such as Organization Administrator) can
        change a developer's status using this API call.

        If you set a developer's status to inactive, the API keys assigned to
        the developer's apps are no longer valid even though keys continue to
        show a status of "Approved"
        (in strikethrough text in the management UI).
        Inactive developers, however, can still log into the developer portal
        and create apps. The new keys that get created just won't work.

        Args:
            action (str): Set to ``active`` or ``inactive``.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}?action={action}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/octet-stream"},
            self._auth,
        )
        resp = requests.post(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def update_developer(self, request_body):
        """Update an existing developer profile.

        To add new values or update existing values, submit the new or updated
        portion of the developer profile along with the rest of the developer
        profile, even if no values are changing.

        To delete attributes from a developer profile, submit the entire profile
        without the attributes that you want to delete.

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.put(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def get_developer_attribute(self, attribute_name):
        """Returns the value of a developer attribute.

        Args:
            attribute_name (str, optional): The attribute name.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}/attributes/{attribute_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def update_a_developer_attribute(self, attribute_name, updated_value):
        """Updates the value of a developer attribute.

        Args:
            attribute_name (str, optional): The attribute name to update.
            updated_value (str, optional): The new attribute value.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}/attributes/{attribute_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        body = {"value": updated_value}
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp

    def delete_developer_attribute(self, attribute_name):
        """Deletes a developer attribute.

        Args:
            attribute_name (str, optional): The attribute name to delete.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}/attributes/{attribute_name}"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.delete(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def get_all_developer_attributes(self):
        """Returns a list of all developer attributes.

        Args:
            None

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}/attributes"
        hdrs = authorization.set_header({"Accept": "application/json"}, self._auth)
        resp = requests.get(uri, headers=hdrs)
        resp.raise_for_status()
        return resp

    def update_all_developer_attributes(self, request_body):
        """Updates or creates developer attributes.

        This API replaces the current list of attributes with the attributes
        specified in the request body.
        This lets you update existing attributes, add new attributes, or delete
        existing attributes by omitting them from the request body.

        Args:
            request_body (str): JSON string.

        Returns:
            requests.Response()
        """
        uri = f"{APIGEE_ADMIN_API_URL}/v1/organizations/{self._org_name}/developers/{self._developer_email}/attributes"
        hdrs = authorization.set_header(
            {"Accept": "application/json", "Content-Type": "application/json"},
            self._auth,
        )
        body = json.loads(request_body)
        resp = requests.post(uri, headers=hdrs, json=body)
        resp.raise_for_status()
        return resp



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/stats"""

import json
from abc import ABC, abstractmethod


class IStats:
    def __init__(self, auth, org_name):
        self._auth = auth
        self._org_name = org_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @abstractmethod
    def get_license_utilization(
        self,
        time_range,
        environments=["test", "prod"],
        tzo=None,
        api_calls_per_year=10000000000,
    ):
        pass


class StatsSerializer:
    def serialize_details(self, stats, format, prefix=None):
        resp = stats
        if format == "text":
            return stats.text
        stats = stats.json()
        if prefix:
            stats = [stat for stat in stats if stat.startswith(prefix)]
        if format == "json":
            return json.dumps(stats)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import argparse
import os


class ArgumentType:
    @staticmethod
    def isfile(f):
        if os.path.isfile(f):
            return f
        raise argparse.ArgumentTypeError("not a file")

    @staticmethod
    def isdir(d):
        if os.path.isdir(d):
            return d
        raise argparse.ArgumentTypeError("not a directory")

    @staticmethod
    def ispath(f):
        if os.path.exists(f):
            return f
        raise argparse.ArgumentTypeError("not a valid path")


class Struct:
    def __init__(self, **entries):
        self.__dict__.update(entries)



import argparse

from apigee.api.targetservers import Targetservers

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.file_parser import FileParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserTargetservers:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_targetservers = self._parser.add_parser(
            "targetservers", aliases=["ts"], help="manage target servers"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._file_parser = kwargs.get("file_parser", FileParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_targetservers(self):
        return self._parser_targetservers

    @parser_targetservers.setter
    def parser_targetservers(self, value):
        self._parser_targetservers = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def file_parser(self):
        return self._file_parser

    @file_parser.setter
    def file_parser(self, value):
        self._file_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_create_a_targetserver_argument(self):
        create_a_targetserver = self._parser_targetservers.add_parser(
            "create",
            aliases=["create-a-targetserver"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Create a TargetServer in the specified environment. TargetServers are used to decouple TargetEndpoint HTTPTargetConnections from concrete URLs for backend services.",
        )
        create_a_targetserver.add_argument(
            "-b", "--body", help="request body", required=True
        )
        create_a_targetserver.set_defaults(
            func=lambda args: console.log(
                Targetservers(args, args.org, args.name)
                .create_a_targetserver(args.environment, args.body)
                .text
            )
        )

    def _build_delete_a_targetserver_argument(self):
        delete_a_targetserver = self._parser_targetservers.add_parser(
            "delete",
            aliases=["delete-a-targetserver"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Delete a TargetServer configuration from an environment. Returns information about the deleted TargetServer.",
        )
        delete_a_targetserver.add_argument("-n", "--name", help="name", required=True)
        delete_a_targetserver.set_defaults(
            func=lambda args: console.log(
                Targetservers(args, args.org, args.name)
                .delete_a_targetserver(args.environment)
                .text
            )
        )

    def _build_list_targetservers_in_an_environment_argument(self):
        list_targetservers_in_an_environment = self._parser_targetservers.add_parser(
            "list",
            aliases=["list-targetservers-in-an-environment"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._prefix_parser(),
            ],
            help="List all TargetServers in an environment.",
        )
        list_targetservers_in_an_environment.set_defaults(
            func=lambda args: console.log(
                Targetservers(
                    args, args.org, None
                ).list_targetservers_in_an_environment(
                    args.environment, prefix=args.prefix
                )
            )
        )

    def _build_get_targetserver_argument(self):
        get_targetserver = self._parser_targetservers.add_parser(
            "get",
            aliases=["get-targetserver"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Returns a TargetServer definition.",
        )
        get_targetserver.add_argument("-n", "--name", help="name", required=True)
        get_targetserver.set_defaults(
            func=lambda args: console.log(
                Targetservers(args, args.org, args.name)
                .get_targetserver(args.environment)
                .text
            )
        )

    def _build_update_a_targetserver_argument(self):
        update_a_targetserver = self._parser_targetservers.add_parser(
            "update",
            aliases=["update-a-targetserver"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Modifies an existing TargetServer.",
        )
        update_a_targetserver.add_argument("-n", "--name", help="name", required=True)
        update_a_targetserver.add_argument(
            "-b", "--body", help="request body", required=True
        )
        update_a_targetserver.set_defaults(
            func=lambda args: console.log(
                Targetservers(args, args.org, args.name)
                .update_a_targetserver(args.environment, args.body)
                .text
            )
        )

    def _build_push_targetserver_argument(self):
        push_targetserver = self._parser_targetservers.add_parser(
            "push",
            aliases=["push-targetserver"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
                self._file_parser(),
            ],
            help="Push TargetServer to Apigee. This will create/update a TargetServer.",
        )
        push_targetserver.set_defaults(
            func=lambda args: Targetservers(args, args.org, None).push_targetserver(
                args.environment, args.file
            )
        )

    def _create_parser(self):
        self._build_create_a_targetserver_argument()
        self._build_delete_a_targetserver_argument()
        self._build_list_targetservers_in_an_environment_argument()
        self._build_get_targetserver_argument()
        self._build_update_a_targetserver_argument()
        self._build_push_targetserver_argument()



#!/usr/bin/env python
"""https://docs.apigee.com/api-platform/system-administration/permissions"""

from abc import ABC, abstractmethod

# import pandas as pd
# from pandas.io.json import json_normalize
from tabulate import tabulate


class IPermissions:
    def __init__(self, auth, org_name, role_name):
        self._auth = auth
        self._org_name = org_name
        self._role_name = role_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def role_name(self):
        return self._role_name

    @role_name.setter
    def role_name(self, value):
        self._role_name = value

    @abstractmethod
    def create_permissions(self, request_body):
        pass

    @abstractmethod
    def team_permissions(
        self, template_file, placeholder_key=None, placeholder_value=""
    ):
        pass

    @abstractmethod
    def get_permissions(
        self, formatted=False, format="text", showindex=False, tablefmt="plain"
    ):
        pass


class PermissionsSerializer:
    def serialize_details(
        self, permission_details, format, showindex=False, tablefmt="plain"
    ):
        if format == "text":
            return permission_details.text
        elif format == "table":
            # pd.set_option('display.max_colwidth', max_colwidth)
            # return pd.DataFrame.from_dict(json_normalize(permission_details.json()['resourcePermission']), orient='columns')
            table = [
                [res["organization"], res["path"], res["permissions"]]
                for res in permission_details.json()["resourcePermission"]
            ]
            headers = []
            if showindex == "always" or showindex is True:
                headers = ["id", "organization", "path", "permissions"]
            elif showindex == "never" or showindex is False:
                headers = ["organization", "path", "permissions"]
            return tabulate(table, headers, showindex=showindex, tablefmt=tablefmt)
        # else:
        #     raise ValueError(format)
        return permission_details



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/user-roles"""

from abc import ABC, abstractmethod


class IUserroles:
    def __init__(self, auth, org_name, role_name):
        self._auth = auth
        self._org_name = org_name
        self._role_name = role_name

    def __call__(self):
        pass

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def role_name(self):
        return self._role_name

    @role_name.setter
    def role_name(self, value):
        self._role_name = value

    @abstractmethod
    def add_a_user_to_a_role(self, user_email):
        pass

    @abstractmethod
    def add_permissions_for_a_resource_to_a_user_role(self, request_body):
        pass

    @abstractmethod
    def add_permissions_for_multiple_resources_to_a_user_role(self, request_body):
        pass

    @abstractmethod
    def create_a_user_role_in_an_organization(self):
        pass

    @abstractmethod
    def delete_a_permission_for_a_resource(self, permission, resource_path):
        pass

    @abstractmethod
    def delete_resource_from_permissions(self, resource_path):
        pass

    @abstractmethod
    def delete_a_user_role(self):
        pass

    @abstractmethod
    def get_a_role(self):
        pass

    @abstractmethod
    def get_resource_permissions_for_a_specific_role(self, resource_path=""):
        pass

    @abstractmethod
    def get_users_for_a_role(self):
        pass

    @abstractmethod
    def list_permissions_for_a_resource(self):
        pass

    @abstractmethod
    def list_user_roles(self):
        pass

    @abstractmethod
    def remove_user_membership_in_role(self, user_email):
        pass

    @abstractmethod
    def verify_a_user_roles_permission_on_a_specific_RBAC_resource(
        self, permission, resource_path
    ):
        pass

    @abstractmethod
    def verify_user_role_membership(self, user_email):
        pass



import argparse

from apigee.api.userroles import Userroles

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserUserroles:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_userroles = self._parser.add_parser(
            "userroles", aliases=["roles"], help="manage user roles"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_userroles(self):
        return self._parser_userroles

    @parser_userroles.setter
    def parser_userroles(self, value):
        self._parser_userroles = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    def __call__(self):
        return self._parser

    def _build_add_a_user_to_a_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "add-user",
            aliases=["add-a-user-to-a-role"],
            help="Add a user to a role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument(
            "--user-email", help="the email address of the user", required=True
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .add_a_user_to_a_role(args.user_email)
                .text
            )
        )

    def _build_add_permissions_for_a_resource_to_a_user_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "add-permissions",
            aliases=["add-permissions-for-a-resource-to-a-user-role"],
            help="Add Permissions for Resource to a Role",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument("-b", "--body", help="request body", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .add_permissions_for_a_resource_to_a_user_role(args.body)
                .text
            )
        )

    def _build_add_permissions_for_multiple_resources_to_a_user_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "add-permissions-multiple",
            aliases=["add-permissions-for-multiple-resources-to-a-user-role"],
            help="Adds multiple permissions to multiple resources simultaneously.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="name", required=True)
        parser.add_argument("-b", "--body", help="request body", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .add_permissions_for_multiple_resources_to_a_user_role(args.body)
                .text
            )
        )

    def _build_create_a_user_role_in_an_organization_argument(self):
        parser = self._parser_userroles.add_parser(
            "create",
            aliases=["create-a-user-role-in-an-organization"],
            help="Creates one ore more user roles in an organization.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument(
            "-n", "--name", nargs="+", help="list of role names", required=True
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .create_a_user_role_in_an_organization()
                .text
            )
        )

    def _build_delete_a_permission_for_a_resource_argument(self):
        parser = self._parser_userroles.add_parser(
            "delete-permission",
            aliases=["delete-a-permission-for-a-resource"],
            help="Removes a permission from a resource for the role specified. Permissions are case sensitive. Specify the permission as get, put, or delete.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument("--permission", help="get, put, or delete", required=True)
        parser.add_argument("--resource-path", help="the resource path", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .delete_a_permission_for_a_resource(args.permission, args.resource_path)
                .text
            )
        )

    def _build_delete_resource_from_permissions_argument(self):
        parser = self._parser_userroles.add_parser(
            "delete-resource",
            aliases=["delete-resource-from-permissions"],
            help="Removes all permissions for a resource for the role specified. Permissions are case sensitive.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument("--resource-path", help="the resource path", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .delete_resource_from_permissions(args.resource_path)
                .text
            )
        )

    def _build_delete_a_user_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "delete",
            aliases=["delete-a-user-role"],
            help="Deletes a role from an organization. Roles can only be deleted when no users are in the role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name).delete_a_user_role().text
            )
        )

    def _build_get_a_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "get",
            aliases=["get-a-role"],
            help="Gets the name of a user role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name).get_a_role().text
            )
        )

    def _build_get_resource_permissions_for_a_specific_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "get-permissions",
            aliases=["get-resource-permissions-for-a-specific-role"],
            help="Gets a list of permissions associated with the specified resource.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument("--resource-path", help="the resource path", default="")
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .get_resource_permissions_for_a_specific_role(
                    resource_path=args.resource_path
                )
                .text
            )
        )

    def _build_get_users_for_a_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "get-users",
            aliases=["get-users-for-a-role"],
            help="Returns a list of all system users associated with a role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name).get_users_for_a_role().text
            )
        )

    def _build_list_permissions_for_a_resource_argument(self):
        parser = self._parser_userroles.add_parser(
            "list-permissions",
            aliases=["list-permissions-for-a-resource"],
            help="Gets permissions for all resources associated with a user role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .list_permissions_for_a_resource()
                .text
            )
        )

    def _build_list_user_roles_argument(self):
        parser = self._parser_userroles.add_parser(
            "list",
            aliases=["list-user-roles"],
            help="Gets a list of roles available to users in an organization.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, None).list_user_roles().text
            )
        )

    def _build_remove_user_membership_in_role_argument(self):
        parser = self._parser_userroles.add_parser(
            "remove-user",
            aliases=["remove-user-membership-in-role"],
            help="Remove user membership in role.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument(
            "--user-email", help="the email address of the user", required=True
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .remove_user_membership_in_role(args.user_email)
                .text
            )
        )

    def _build_verify_a_user_roles_permission_on_a_specific_RBAC_resource_argument(
        self,
    ):
        parser = self._parser_userroles.add_parser(
            "verify-permission",
            aliases=["verify-a-user-roles-permission-on-a-specific-RBAC-resource"],
            help="Verifies that a user role's permission on a specific resource exists. Returns a value of true or false.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument("--permission", help="get, put, or delete", required=True)
        parser.add_argument("--resource-path", help="the resource path", required=True)
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .verify_a_user_roles_permission_on_a_specific_RBAC_resource(
                    args.permission, args.resource_path
                )
                .text
            )
        )

    def _build_verify_user_role_membership_argument(self):
        parser = self._parser_userroles.add_parser(
            "verify-membership",
            aliases=["verify-user-role-membership"],
            help="Verify user role membership.",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
        )
        parser.add_argument("-n", "--name", help="the role name", required=True)
        parser.add_argument(
            "--user-email", help="the email address of the user", required=True
        )
        parser.set_defaults(
            func=lambda args: console.log(
                Userroles(args, args.org, args.name)
                .verify_user_role_membership(args.user_email)
                .text
            )
        )

    def _create_parser(self):
        self._build_add_a_user_to_a_role_argument()
        self._build_add_permissions_for_a_resource_to_a_user_role_argument()
        self._build_add_permissions_for_multiple_resources_to_a_user_role_argument()
        self._build_create_a_user_role_in_an_organization_argument()
        self._build_delete_a_permission_for_a_resource_argument()
        self._build_delete_resource_from_permissions_argument()
        self._build_delete_a_user_role_argument()
        self._build_get_a_role_argument()
        self._build_get_resource_permissions_for_a_specific_role_argument()
        self._build_get_users_for_a_role_argument()
        self._build_list_permissions_for_a_resource_argument()
        self._build_list_user_roles_argument()
        self._build_remove_user_membership_in_role_argument()
        self._build_verify_a_user_roles_permission_on_a_specific_RBAC_resource_argument()
        self._build_verify_user_role_membership_argument()



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/apps-developer

https://apidocs.apigee.com/api/developer-app-keys
"""

import json
from abc import ABC, abstractmethod


class IApps:
    def __init__(self, auth, org_name, app_name):
        self._auth = auth
        self._org_name = org_name
        self._app_name = app_name

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def app_name(self):
        return self._app_name

    @app_name.setter
    def app_name(self, value):
        self._app_name = value

    def __call__(self):
        pass

    @abstractmethod
    def create_developer_app(self, developer, request_body):
        pass

    @abstractmethod
    def create_empty_developer_app(self, developer, display_name="", callback_url=""):
        pass

    @abstractmethod
    def delete_developer_app(self, developer):
        pass

    @abstractmethod
    def get_developer_app_details(self, developer):
        pass

    @abstractmethod
    def list_developer_apps(
        self, developer, prefix=None, expand=False, count=100, startkey=""
    ):
        pass

    @abstractmethod
    def delete_key_for_a_developer_app(self, developer, consumer_key):
        pass

    @abstractmethod
    def create_a_consumer_key_and_secret(
        self,
        developer,
        consumer_key=None,
        consumer_secret=None,
        key_length=32,
        secret_length=32,
        key_suffix=None,
        key_delimiter="-",
        products=[],
    ):
        pass

    @abstractmethod
    def add_api_product_to_key(self, developer, consumer_key, request_body):
        pass

    @abstractmethod
    def restore_app(self, file):
        pass


class AppsSerializer:
    def serialize_details(self, apps, format, prefix=None):
        resp = apps
        if format == "text":
            return apps.text
        apps = apps.json()
        if prefix:
            apps = [app for app in apps if app.startswith(prefix)]
        if format == "json":
            return json.dumps(apps)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



#!/usr/bin/env python
"""https://apidocs.apigee.com/api/developers-0"""

import json
from abc import ABC, abstractmethod


class IDevelopers:
    def __init__(self, auth, org_name, developer_email):
        self._auth = auth
        self._org_name = org_name
        self._developer_email = developer_email

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def developer_email(self):
        return self._developer_email

    @developer_email.setter
    def developer_email(self, value):
        self._developer_email = value

    def __call__(self):
        pass

    @abstractmethod
    def create_developer(
        self, first_name, last_name, user_name, attributes='{"attributes" : [ ]}'
    ):
        pass

    @abstractmethod
    def delete_developer(self):
        pass

    @abstractmethod
    def get_developer(self):
        pass

    @abstractmethod
    def get_developer_by_app(self, app_name):
        pass

    @abstractmethod
    def list_developers(self):
        pass

    @abstractmethod
    def set_developer_status(self, action):
        pass

    @abstractmethod
    def update_developer(self, request_body):
        pass

    @abstractmethod
    def get_developer_attribute(self, attribute_name):
        pass

    @abstractmethod
    def update_a_developer_attribute(self, attribute_name, updated_value):
        pass

    @abstractmethod
    def delete_developer_attribute(self, attribute_name):
        pass

    @abstractmethod
    def get_all_developer_attributes(self):
        pass

    @abstractmethod
    def update_all_developer_attributes(self, request_body):
        pass


class DevelopersSerializer:
    def serialize_details(self, developers, format, prefix=None):
        resp = developers
        if format == "text":
            return developers.text
        developers = developers.json()
        if prefix:
            developers = [
                developer for developer in developers if developer.startswith(prefix)
            ]
        if format == "json":
            return json.dumps(developers)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



#!/usr/bin/env python
"""https://apidocs.apigee.com/api-reference/content/data-masks"""

import json
from abc import ABC, abstractmethod


class IMaskconfigs:
    def __init__(self, auth, org_name, api_name):
        self._auth = auth
        self._org_name = org_name
        self._api_name = api_name

    @property
    def auth(self):
        return self._auth

    @auth.setter
    def auth(self, value):
        self._auth = value

    @property
    def org_name(self):
        return self._org_name

    @org_name.setter
    def org_name(self, value):
        self._org_name = value

    @property
    def api_name(self):
        return self._api_name

    @api_name.setter
    def api_name(self, value):
        self._api_name = value

    def __call__(self):
        pass

    @abstractmethod
    def create_data_masks_for_an_api_proxy(self, request_body):
        pass

    @abstractmethod
    def delete_data_masks_for_an_api_proxy(self, maskconfig_name):
        pass

    @abstractmethod
    def get_data_mask_details_for_an_api_proxy(self, maskconfig_name):
        pass

    @abstractmethod
    def list_data_masks_for_an_api_proxy(self):
        pass

    @abstractmethod
    def list_data_masks_for_an_organization(self):
        pass

    @abstractmethod
    def push_data_masks_for_an_api_proxy(self, file):
        pass


class MaskconfigsSerializer:
    def serialize_details(self, maskconfigs, format, prefix=None):
        resp = maskconfigs
        if format == "text":
            return maskconfigs.text
        maskconfigs = maskconfigs.json()
        if prefix:
            maskconfigs = [
                maskconfig
                for maskconfig in maskconfigs
                if maskconfig.startswith(prefix)
            ]
        if format == "json":
            return json.dumps(maskconfigs)
        elif format == "table":
            pass
        # else:
        #     raise ValueError(format)
        return resp



import base64
import configparser
import os
import time

import jwt

from apigee import *
from apigee.api.developers import Developers
from apigee.util import mfa_with_pyotp
from apigee.util.os import makedirs
from apigee.util.types import Struct


def set_header(hdrs, args):
    if hdrs is None:
        hdrs = {}
    if args.mfa_secret:
        access_token = ""
        makedirs(APIGEE_CLI_DIRECTORY)
        try:
            with open(APIGEE_CLI_ACCESS_TOKEN_FILE, "r") as f:
                access_token = f.read()
        except (IOError, OSError) as e:
            pass
        finally:
            if access_token:
                decoded = jwt.decode(access_token, verify=False)
                if (
                    decoded["exp"] < int(time.time())
                    or decoded["email"] != args.username
                ):
                    access_token = ""
        if not access_token:
            access_token = mfa_with_pyotp.get_access_token(args)
            with open(APIGEE_CLI_ACCESS_TOKEN_FILE, "w") as f:
                f.write(access_token)
        hdrs["Authorization"] = f"Bearer {access_token}"
    else:
        cred = base64.b64encode((f"{args.username}:{args.password}").encode()).decode()
        hdrs["Authorization"] = f"Basic {cred}"
    return hdrs


def get_credential(section, key):
    try:
        config = configparser.ConfigParser()
        config.read(APIGEE_CLI_CREDENTIALS_FILE)
        if section in config:
            return config[section][key]
    except:
        return


def with_prefix(
    name, args, attribute_name=APIGEE_CLI_AUTHORIZATION_DEVELOPER_ATTRIBUTE
):
    team = (
        Developers(args, args.org, args.username)
        .get_developer_attribute(attribute_name)
        .json()["value"]
    )
    allowed = team.split(",")
    for prefix in allowed:
        if name.startswith(prefix):
            return name
    raise Exception(
        f"401 Client Error: Unauthorized for team: {str(allowed)}\nAttempted to access resource: {name}"
    )


def gen_auth(username, password, mfa_secret):
    return Struct(username=username, password=password, mfa_secret=mfa_secret)



import argparse

from apigee.api import deploy
from apigee.api.apis import Apis

from apigee.parsers.parent_parser import ParentParser
from apigee.parsers.dir_parser import DirParser
from apigee.parsers.environment_parser import EnvironmentParser
from apigee.parsers.prefix_parser import PrefixParser
from apigee.parsers.silent_parser import SilentParser
from apigee.parsers.verbose_parser import VerboseParser

from apigee.util import console


class ParserApis:
    def __init__(self, parser, **kwargs):
        self._parser = parser
        self._parser_apis = self._parser.add_parser(
            "apis", help="manage APIs"
        ).add_subparsers()
        self._parent_parser = kwargs.get("parent_parser", ParentParser())
        self._dir_parser = kwargs.get("dir_parser", DirParser())
        self._environment_parser = kwargs.get("environment_parser", EnvironmentParser())
        self._prefix_parser = kwargs.get(
            "prefix_parser", PrefixParser(profile="default")
        )
        self._silent_parser = kwargs.get("silent_parser", SilentParser())
        self._verbose_parser = kwargs.get("verbose_parser", VerboseParser())
        self._create_parser()

    @property
    def parser(self):
        return self._parser

    @parser.setter
    def parser(self, value):
        self._parser = value

    @property
    def parser_apis(self):
        return self._parser_apis

    @parser_apis.setter
    def parser_apis(self, value):
        self._parser_apis = value

    @property
    def parent_parser(self):
        return self._parent_parser

    @parent_parser.setter
    def parent_parser(self, value):
        self._parent_parser = value

    @property
    def dir_parser(self):
        return self._dir_parser

    @dir_parser.setter
    def dir_parser(self, value):
        self._dir_parser = value

    @property
    def environment_parser(self):
        return self._environment_parser

    @environment_parser.setter
    def environment_parser(self, value):
        self._environment_parser = value

    @property
    def prefix_parser(self):
        return self._prefix_parser

    @prefix_parser.setter
    def prefix_parser(self, value):
        self._prefix_parser = value

    def __call__(self):
        return self._parser

    def _build_apis_deploy_argument(self):
        apis_deploy = self._parser_apis.add_parser(
            "deploy",
            help="deploy apis",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._dir_parser(),
                self._environment_parser(),
            ],
        )
        apis_deploy.add_argument("-n", "--name", help="name", required=True)
        # apis_deploy.add_argument('-d', '--directory', help='directory name')
        # apis_deploy.add_argument('-p', '--path', help='base path')
        apis_deploy.add_argument(
            "-i",
            "--import-only",
            action="store_true",
            help="denotes to import only and not actually deploy",
        )
        apis_deploy.add_argument(
            "-s",
            "--seamless-deploy",
            action="store_true",
            help="seamless deploy the bundle",
        )
        apis_deploy.set_defaults(func=deploy.deploy)

    def _build_delete_api_proxy_revision_argument(self):
        delete_api_proxy_revision = self._parser_apis.add_parser(
            "delete-rev",
            aliases=["delete-api-proxy-revision", "delete-revision"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Deletes a revision of an API proxy and all policies, resources, endpoints, and revisions associated with it. The API proxy revision must be undeployed before you can delete it.",
        )
        delete_api_proxy_revision.add_argument(
            "-n", "--name", help="name", required=True
        )
        delete_api_proxy_revision.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        delete_api_proxy_revision.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org)
                .delete_api_proxy_revision(args.name, args.revision_number)
                .text
            )
        )

    def _build_deploy_api_proxy_revision_argument(self):
        deploy_api_proxy_revision = self._parser_apis.add_parser(
            "deploy-rev",
            aliases=["deploy-api-proxy-revision", "deploy-revision"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Deploys a revision of an existing API proxy to an environment in an organization.",
        )
        deploy_api_proxy_revision.add_argument(
            "-n", "--name", help="name", required=True
        )
        deploy_api_proxy_revision.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        deploy_api_proxy_revision.add_argument(
            "--delay",
            type=int,
            default=0,
            help="Enforces a delay, measured in seconds, before the currently deployed API proxy revision is undeployed and replaced by the new revision that is being deployed. Use this setting to minimize the impact of deployment on in-flight transactions. The default value is 0.",
        )
        deploy_api_proxy_revision.add_argument(
            "--override",
            action="store_true",
            help='Flag that specifies whether to use seamless deployment to ensure zero downtime. Set this flag to "true" to instruct Edge to deploy the new revision fully before undeploying the existing revision. Use in conjunction with the delay parameter to control when the existing revision is undeployed.',
        )
        deploy_api_proxy_revision.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org)
                .deploy_api_proxy_revision(
                    args.name,
                    args.environment,
                    args.revision_number,
                    delay=args.delay,
                    override=args.override,
                )
                .text
            )
        )

    def _build_delete_undeployed_revisions_argument(self):
        delete_undeployed_revisions = self._parser_apis.add_parser(
            "clean",
            aliases=["delete-undeployed-revisions"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Deletes all undeployed revisions of an API proxy and all policies, resources, endpoints, and revisions associated with it.",
        )
        delete_undeployed_revisions.add_argument(
            "-n", "--name", help="name", required=True
        )
        delete_undeployed_revisions.add_argument(
            "--save-last",
            metavar="N",
            type=int,
            default=0,
            help="denotes not to delete the N most recent revisions",
            required=False,
        )
        delete_undeployed_revisions.add_argument(
            "--dry-run",
            action="store_true",
            help="show revisions to be deleted but do not delete",
            required=False,
        )
        delete_undeployed_revisions.set_defaults(
            func=lambda args: Apis(args, args.org).delete_undeployed_revisions(
                args.name, save_last=args.save_last, dry_run=args.dry_run
            )
        )

    def _build_export_api_proxy_argument(self):
        export_api_proxy = self._parser_apis.add_parser(
            "export",
            aliases=["export-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Outputs an API proxy revision as a ZIP formatted bundle of code and config files. This enables local configuration and development, including attachment of policies and scripts.",
        )
        export_api_proxy.add_argument("-n", "--name", help="name", required=True)
        export_api_proxy.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        export_api_proxy.add_argument("-O", "--output-file", help="output file")
        export_api_proxy.set_defaults(
            func=lambda args: Apis(args, args.org).export_api_proxy(
                args.name,
                args.revision_number,
                fs_write=True,
                output_file=args.output_file
                if args.output_file
                else f"{args.name}.zip",
            )
        )
        # export_api_proxy.set_defaults(func=apis.export_api_proxy)

    def _build_pull_argument(self):
        pull_api = self._parser_apis.add_parser(
            "pull",
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Pull API proxy revision as a ZIP formatted bundle along with KeyValueMap and TargetServer dependencies into the current working directory.",
        )
        pull_api.add_argument("-n", "--name", help="name", required=True)
        pull_api.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        pull_api.add_argument(
            "-f", "--force", action="store_true", help="force write files"
        )
        pull_api.add_argument("--work-tree", help="set the path to the working tree")
        pull_api.add_argument(
            "--prefix",
            help="prefix to prepend. WARNING: this is not foolproof. make sure to review the changes.",
        )
        pull_api.add_argument(
            "-b",
            "--basepath",
            help="set default basepath in apiproxy/proxies/default.xml",
        )
        pull_api.set_defaults(
            func=lambda args: Apis(
                args,
                args.org,
                args.revision_number,
                args.environment,
                work_tree=args.work_tree,
            ).pull(
                args.name, force=args.force, prefix=args.prefix, basepath=args.basepath
            )
        )

    def _build_get_api_proxy_argument(self):
        get_api_proxy = self._parser_apis.add_parser(
            "get",
            aliases=["get-api-proxy"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="Gets an API proxy by name, including a list of existing revisions of the proxy.",
        )
        get_api_proxy.add_argument("-n", "--name", help="name", required=True)
        get_api_proxy.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org).get_api_proxy(args.name).text
            )
        )

    def _build_list_api_proxies_argument(self):
        list_api_proxies = self._parser_apis.add_parser(
            "list",
            aliases=["list-api-proxies"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._prefix_parser(),
            ],
            help="Gets the names of all API proxies in an organization. The names correspond to the names defined in the configuration files for each API proxy.",
        )
        list_api_proxies.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org).list_api_proxies(prefix=args.prefix)
            )
        )

    def _build_list_api_proxy_revisions_argument(self):
        list_api_proxies = self._parser_apis.add_parser(
            "list-revs",
            aliases=["list-api-proxy-revisions", "list-revisions"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
            ],
            help="List all revisions for an API proxy.",
        )
        list_api_proxies.add_argument("-n", "--name", help="name", required=True)
        list_api_proxies.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org).list_api_proxy_revisions(args.name).text
            )
        )

    def _build_undeploy_api_proxy_revision_argument(self):
        undeploy_api_proxy_revision = self._parser_apis.add_parser(
            "undeploy-rev",
            aliases=["undeploy-api-proxy-revision", "undeploy-revision"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Undeploys an API proxy revision from an environment.",
        )
        undeploy_api_proxy_revision.add_argument(
            "-n", "--name", help="name", required=True
        )
        undeploy_api_proxy_revision.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        undeploy_api_proxy_revision.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org)
                .undeploy_api_proxy_revision(
                    args.name, args.environment, args.revision_number
                )
                .text
            )
        )

    def _build_force_undeploy_api_proxy_revision_argument(self):
        force_undeploy_api_proxy_revision = self._parser_apis.add_parser(
            "force-undeploy-rev",
            aliases=["force-undeploy-api-proxy-revision", "force-undeploy-revision"],
            parents=[
                self._parent_parser(),
                self._silent_parser(),
                self._verbose_parser(),
                self._environment_parser(),
            ],
            help="Force the undeployment of the API proxy that is partially deployed.",
        )
        force_undeploy_api_proxy_revision.add_argument(
            "-n", "--name", help="name", required=True
        )
        force_undeploy_api_proxy_revision.add_argument(
            "-r", "--revision-number", type=int, help="revision number", required=True
        )
        force_undeploy_api_proxy_revision.set_defaults(
            func=lambda args: console.log(
                Apis(args, args.org)
                .force_undeploy_api_proxy_revision(
                    args.name, args.environment, args.revision_number
                )
                .text
            )
        )

    def _create_parser(self):
        self._build_apis_deploy_argument()
        self._build_delete_api_proxy_revision_argument()
        self._build_deploy_api_proxy_revision_argument()
        self._build_delete_undeployed_revisions_argument()
        self._build_export_api_proxy_argument()
        self._build_pull_argument()
        self._build_get_api_proxy_argument()
        self._build_list_api_proxies_argument()
        self._build_list_api_proxy_revisions_argument()
        self._build_undeploy_api_proxy_revision_argument()
        self._build_force_undeploy_api_proxy_revision_argument()
